import json
from pathlib import Path
from typing import List, Optional, Set

from nonebot import get_plugin_config
from nonebot.compat import type_validate_json
from pydantic import BaseModel, FilePath


class ConfigModel(BaseModel):
    fuckyou_gentle: bool = True
    fuckyou_violent: bool = False
    fuckyou_use_builtin_gentle: bool = True
    fuckyou_use_builtin_violent: bool = True
    fuckyou_external_gentle_path: Optional[FilePath] = None
    fuckyou_external_violent_path: Optional[FilePath] = None

    fuckyou_use_builtin_words: bool = True
    fuckyou_extend_words: Set[str] = set()
    fuckyou_tome: bool = True

    fuckyou_user_blacklist: Set[str] = set()
    fuckyou_user_bl_to_wl: bool = False
    fuckyou_group_blacklist: Set[str] = set()
    fuckyou_group_bl_to_wl: bool = False

    fuckyou_block: bool = False
    fuckyou_inspect: bool = False


config = get_plugin_config(ConfigModel)


def load_external_phase(p: Path):
    return type_validate_json(List[str], json.loads(p.read_text("u8")))



from nonebot.plugin import PluginMetadata, inherit_supported_adapters, require

require("nonebot_plugin_alconna")
require("nonebot_plugin_session")

from .__main__ import get_phase  # noqa: E402
from .config import ConfigModel  # noqa: E402

__version__ = "0.3.0"
__plugin_meta__ = PluginMetadata(
    name="FuckYou",
    description=get_phase(),
    usage="与机器人对骂即可（关键词触发）",
    homepage="https://github.com/lgc-NB2Dev/nonebot-plugin-fuckyou",
    type="application",
    config=ConfigModel,
    supported_adapters=inherit_supported_adapters(
        "nonebot_plugin_alconna",
        "nonebot_plugin_session",
    ),
    extra={"License": "MIT", "Author": "student_2333"},
)



import json
from pathlib import Path
from typing import List, Set

RESOURCE_DIR = Path(__file__).parent / "resource"
BUILTIN_GENTLE_PATH = RESOURCE_DIR / "builtin_gentle.json"
BUILTIN_VIOLENT_PATH = RESOURCE_DIR / "builtin_violent.json"

GENTLE: List[str] = json.loads(BUILTIN_GENTLE_PATH.read_text(encoding="u8"))
VIOLENT: List[str] = json.loads(BUILTIN_VIOLENT_PATH.read_text(encoding="u8"))

DEFAULT_TRIGGER_WORDS: Set[str] = {
    "傻逼",
    "你妈逼",
    "日你妈",
    "草你妈",
    "操你妈",
    "cnm",
    "煞笔",
    "啥比",
    "杀币",
    "鲨臂",
    "shabi",
    "sb",
    "我是你爹",
}



import random
from typing import List, Set
from typing_extensions import Annotated

from nonebot import logger, on_message
from nonebot.adapters import Event
from nonebot_plugin_alconna.uniseg import UniMessage
from nonebot_plugin_session import SessionId, SessionIdType

from .config import config, load_external_phase
from .const import DEFAULT_TRIGGER_WORDS, GENTLE, VIOLENT

PHASES: List[str] = [
    *(GENTLE if config.fuckyou_use_builtin_gentle and config.fuckyou_gentle else []),
    *(VIOLENT if config.fuckyou_use_builtin_violent and config.fuckyou_violent else []),
    *(
        load_external_phase(p)
        if (p := config.fuckyou_external_gentle_path) and config.fuckyou_gentle
        else []
    ),
    *(
        load_external_phase(p)
        if (p := config.fuckyou_external_violent_path) and config.fuckyou_violent
        else []
    ),
]
if not PHASES:
    raise ValueError("请至少选择一个骂人词库")

TRIGGER_WORDS: Set[str] = (
    DEFAULT_TRIGGER_WORDS if config.fuckyou_use_builtin_words else set()
) | config.fuckyou_extend_words


GroupID = Annotated[
    str,
    SessionId(
        SessionIdType.GROUP,
        include_bot_type=False,
        include_bot_id=False,
    ),
]


def get_phase() -> str:
    return random.choice(PHASES)


# nb2 的 on_message 貌似不支持忽略大小写，自己写一个
def trigger_rule(event: Event, group_id: GroupID):
    if config.fuckyou_tome and (not event.is_tome()):
        return False

    in_user_blacklist = event.get_user_id() in config.fuckyou_user_blacklist
    if config.fuckyou_user_bl_to_wl:
        in_user_blacklist = not in_user_blacklist
    in_group_blacklist = group_id in config.fuckyou_group_blacklist
    if config.fuckyou_group_bl_to_wl:
        in_group_blacklist = not in_group_blacklist
    if any((in_user_blacklist, in_group_blacklist)):
        return False

    try:
        msg = event.get_plaintext().lower()
    except Exception:
        return False

    return any(w in msg for w in TRIGGER_WORDS)


trigger_matcher = on_message(rule=trigger_rule, block=config.fuckyou_block)


@trigger_matcher.handle()
async def _(group_id: GroupID):
    if config.fuckyou_inspect:
        logger.success(group_id)
    phase = get_phase()
    await UniMessage(phase).send(reply_to=True)
